# -*- coding: utf-8 -*-
import logging
import traceback
import subprocess
import ftplib
import random
import string
import sys
import os
from os.path import abspath
from os.path import dirname
from os.path import isfile
from os.path import join
from six import StringIO
from pkg_resources import resource_exists
from pkg_resources import resource_filename
try:
    from synapselabels import __version__

    assert __version__
except ImportError:
    __version__ = "UNKNOWN"
try:
    from gi import require_version
except ImportError:
    print("ERROR: pygobject is not installed.")
    raise
try:
    require_version("Gtk", "3.0")
except ValueError:
    print("ERROR: gtk3 development files are not installed.")
    raise
except AttributeError:
    print("pygobject version too old.")
    raise
try:
    from gi.repository import Gtk
    from gi.repository import Gdk
    # from gi.repository import GLib
    from gi.repository import GObject
except ImportError:
    print("ERROR: pygobject is not installed.")
    raise


svg_dir = join(dirname(__file__), 'svg')


def get_path(name):
    """
    Returns the absolute path name of a resource file
    File in the current directory overrides file in package
    """
    path = None
    # if isfile(name):
    #     path = abspath(name)
    if __file__ is not None and isfile(join(dirname(__file__), name)):
        path = join(dirname(__file__), name)
    elif resource_exists(__name__, name):
        res_filename = resource_filename(__name__, name)
        res_path = abspath(join(dirname(__file__), res_filename))
        if isfile(res_path):
            path = res_path
    if path:
        logging.info("located path: %s", path)
    else:
        logging.error("could not locate path for: %s", name)
    return path


def list_of_dicts_to_table(data_log):
    params = set()
    for record in data_log:
        for key in record.keys():
            params.add(key)
    params = sorted(list(params))
    print('params:', params)
    table = '\t'.join(params)
    for record in data_log:
        line_items = []
        for param in params:
            line_items.append(record[param] if param in record else '')
        table += '\n' + '\t'.join(line_items)
    return table


def start_file(filepath):
    if sys.platform.startswith('darwin'):
        subprocess.call(('open', filepath))
    elif os.name == 'nt':
        os.startfile(filepath)
    elif os.name == 'posix':
        subprocess.call(('xdg-open', filepath))


def get_selected_data(treeview):
    # print("print_selected_label")
    (model, pathlist) = treeview.get_selection().get_selected_rows()
    for path in pathlist:
        tree_iter = model.get_iter(path)
        # for index, column in enumerate(
        #         treeview.get_columns()):
        #     return model.get_value(tree_iter, index)
        return model.get_value(tree_iter, 0)


def on_drag_data_get(treeview, context, selection, info, timestamp): \
        # pylint: disable=unused-argument
    """ Extract data from the source of the DnD operation.

    Serialize iterators of selected tasks in format
    <iter>,<iter>,...,<iter> and set it as parameter of DND """

    selected_data = get_selected_data(treeview)
    selection.set(selection.get_target(), 0, selected_data)


def set_treeview_columns(treeview, columns):
    for i in range(len(columns)):
        # cellrenderer to render the text
        cell = Gtk.CellRendererText(editable=False)
        # cell.set_property("editable", True)
        # the text in the first column should be in boldface
        # if i == 0:
        #    cell.props.weight_set = True
        #    cell.props.weight = Pango.Weight.BOLD
        # the column is created
        col = Gtk.TreeViewColumn(
            columns[i], cell, text=i)
        # and it is appended to the treeview
        treeview.append_column(col)


def set_treeview_from_clipboard(treeview, with_header=True):
    set_treeview_from_text_data(
        treeview,
        Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD).wait_for_text(),
        with_header
    )


def set_treeview_from_text_data(treeview, text_data, with_header=True):
    listmodel = None
    if text_data is not None:
        # print("clipboard text:", repr(text_data))
        for line in text_data.splitlines():
            # print("line:", repr(line))
            texts = line.split('\t')
            if listmodel is None:
                for column in treeview.get_columns():
                    treeview.remove_column(column)
                args = [GObject.TYPE_STRING] * len(texts)
                # print(args)
                listmodel = Gtk.ListStore(*args)
                # listmodel = Gtk.ListStore(str, str, str)
                if with_header:
                    columns = texts
                else:
                    columns = [
                        "Column{}".format(index) for
                        index in
                        range(0, len(texts))
                    ]
                    # print("appending:", texts)
                    listmodel.append(texts)
            else:
                # print("appending:", texts)
                listmodel.append(texts)
        if listmodel is not None:
            treeview.set_model(listmodel)
            set_treeview_columns(treeview, columns)
    else:
        print("No text on the clipboard.")


def set_treeview_from_texts(treeview, texts):
    # treeview.set_headers_visible(False)
    # print('set treeview from: ', repr(texts))
    listmodel = Gtk.ListStore(GObject.TYPE_STRING)
    if texts is not None:
        for line in texts:
            listmodel.append([line])
    treeview.set_model(listmodel)


def fill_combo(combo, data):
    store = Gtk.ListStore(GObject.TYPE_STRING)
    for line in data:
        store.append([line])
    combo.set_model(store)
    combo.set_active(0)
    # And here's the new stuff:
    # cell = Gtk.CellRendererText()
    # combo.pack_start(cell, True)
    # combo.add_attribute(cell, "text", 0)


def get_treeview_column(treeview, x_pos):
    for column in treeview.get_columns():
        if column.get_x_offset() <= x_pos:
            if x_pos <= (column.get_x_offset() + column.get_width()):
                return column


def drag_data_received(
        widget, context, x_pos, y_pos, selection, info, timestamp): \
        # pylint: disable=unused-argument
    data = selection.get_data()
    print('drag_data_received data:', data)
    if data:
        column = get_treeview_column(widget, x_pos)
        if column:
            print(
                "old column title:", repr(column.get_title()),
                "new column title:", repr(data)
            )
            column.set_title(data)


def print_label_file_to(label_file, printer_config):
    # create random file name
    filename = ''.join(random.choice(string.digits) for _ in range(9))
    # print("random file name:", filename)
    if 'ftp' in printer_config:
        ftp_config = printer_config['ftp']
        ftp_connection = None
        try:
            ftp_connection = ftplib.FTP(
                ftp_config['ip address']
            )
            ftp_connection.login(
                ftp_config['user name'],
                ftp_config['password']
            )
            ftp_connection.cwd("execute")
            ftp_connection.storbinary(
                'STOR ' + filename,
                StringIO(label_file)
            )
        except ftplib.all_errors:
            print("FTP Error")
            traceback.print_exc(file=sys.stdout)
            raise
        finally:
            if ftp_connection is not None:
                ftp_connection.close()
